/*
 * Created on 28.7.2004
 */
package net.sf.tomp.highlight.html;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;

import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import de.java2html.javasource.JavaSource;
import de.java2html.javasource.JavaSourceParser;
import de.java2html.options.Java2HtmlConversionOptions;
import de.java2html.options.JavaSourceStyleTable;

/**
 * @author tomp
 */
public class Java2HTML {

    protected static int TAB_SIZE = 4;

    protected static boolean SHOW_LINE_NUMBERS = true;

    protected static boolean SHOW_TABLE_BORDER = true;

    protected static boolean ADD_LINE_ANCHORS = true;

    protected static String DEFAULT_STYLE_TABLE = "Kawa";

    /**
     * @param src the orig. source code
     * @return Simplest version of java2html with all default options
     */
    public static Source java2html(String src) {
        return java2html(src, SHOW_LINE_NUMBERS, null, DEFAULT_STYLE_TABLE);
    }

    /**
     * @param src the orig. source code
     * @return Simplest version of java2html with all default options
     */
    public static Source java2html(String src, String filename) {
        return java2html(src, SHOW_LINE_NUMBERS, filename, DEFAULT_STYLE_TABLE);
    }

    /**
     * @param src the orig. source code
     * @return Simplest version of java2html with all default options
     */
    public static Source java2html(String src, String filename, String styletable) {
        return java2html(src, SHOW_LINE_NUMBERS, filename, styletable);
    }

    /**
     * @param src the orig. source code
     * @param showLineNumbers whether to show line numbers
     * @param showFileName whether to showFileName
     * @return Simple version of java2html with some default options
     */
    public static Source java2html(String src, boolean showLineNumbers,
            String fileName, String styletable) {
        return java2html(src, showLineNumbers, TAB_SIZE,
                SHOW_TABLE_BORDER, ADD_LINE_ANCHORS, fileName, styletable);
    }

    /**
     * Creates a StreamSource with the highlighted source code src
     * 
     * @param src the orig. source code
     * @param showLineNumbers whether to show line numbers
     * @param tabSize which tab size
     * @param showFileName whether to showFileName
     * @param showTableBorder whether to showTableBorder
     * @param addLineAnchors whether to addLineAnchors
     * @return the StreamSource with the highlighted source code, in case of an
     *         error, the orig. src is returned.
     */
    public static Source java2html(String src, boolean showLineNumbers,
            int tabSize, boolean showTableBorder,
            boolean addLineAnchors, String fileName, String styletable) {

        // set the options
        Java2HtmlConversionOptions options = Java2HtmlConversionOptions
                .getDefault();
        // usedefault style
        options.setStyleTable(JavaSourceStyleTable.getPredefinedTable(styletable));
        // ... and modify it
        options.setShowLineNumbers(showLineNumbers);
        options.setShowFileName(fileName != null);
        options.setTabSize(tabSize);
        options.setShowTableBorder(showTableBorder);
        options.setAddLineAnchors(addLineAnchors);

        // generate Source
        //      Create a reader of the raw input text
        StringReader stringReader = new StringReader(src);

        //        Parse the raw text to a JavaSource object
        JavaSource javaSrc = null;
        try {
            javaSrc = new JavaSourceParser().parse(stringReader);
            javaSrc.setFileName(fileName);
        } catch (IOException e) {
            e.printStackTrace();
        }

        //      Create a converter and write the JavaSource object as XML
        JavaSource2HTMLConverter converter = new JavaSource2HTMLConverter(javaSrc);
        converter.setConversionOptions(options);
        StringWriter writer = new StringWriter();
        try {
            converter.convert(writer);
//            return writer.toString();
            return asSource(writer.toString());
        } catch (IOException e) {
//            return src;
            return asSource(src);
        }
    }

    protected static Source asSource(String src) {
        return new StreamSource(new StringReader(src));
    }
}